/*
 * ${license.title}
 * Copyright (C) 2022 ${license.owner}
 * ${license.mailto}
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <filesystem>
#include <iostream>

#if __has_include("fggl/phys/bullet/bullet.hpp")
	#include "fggl/phys/bullet/bullet.hpp"
#endif

#include "fggl/fggl.hpp"
#include "fggl/entity/module.hpp"

#include "fggl/audio/openal/audio.hpp"

#include "fggl/gfx/atlas.hpp"
#include "fggl/display/glfw/window.hpp"
#include "fggl/gfx/ogl/compat.hpp"

#include "fggl/platform/paths.hpp"

#include "fggl/scenes/menu.hpp"
#include "fggl/modules/manager.hpp"
#include "fggl/assets/module.hpp"

#include "GameScene.h"
#include "rollball.hpp"
#include "topdown.hpp"

static void setup_menu(fggl::App& app) {
	auto *menu = app.addState<fggl::scenes::BasicMenu>("menu");

	// add some menu items for the game states
	menu->add("terrain", [&app]() {
		auto* audio = app.service<fggl::audio::AudioService>();
		audio->play("click.ogg", false);
		app.change_state("game");
	});

	menu->add("rollball", [&app]() {
		auto* audio = app.service<fggl::audio::AudioService>();
		audio->play("click.ogg", false);
		app.change_state("rollball");
	});

	menu->add("Top Down", [&app]() {
		auto* audio = app.service<fggl::audio::AudioService>();
		audio->play("click.ogg", false);
		app.change_state("topdown");
	});

	menu->add("quit", [&app]() {
		auto* audio = app.service<fggl::audio::AudioService>();
		audio->play("click.ogg", false);
		app.running(false);
	});
}

int main(int argc, const char* argv[]) {

	// DAG initialisation test
	fggl::modules::Manager moduleManager;
	moduleManager.use<fggl::input::Generic>();
	moduleManager.use<fggl::data::LocalStorage>();
	moduleManager.use<fggl::gui::FreeType>();
	moduleManager.use<fggl::audio::OpenAL>();
	moduleManager.use<fggl::gfx::OpenGL4>();
	moduleManager.use<fggl::display::GLFW>();
	moduleManager.use<fggl::assets::AssetFolders>();
	moduleManager.use<fggl::entity::ECS>();
	moduleManager.resolve();

	// create the application
	fggl::App app( &moduleManager, "fggl-demo" );

	auto* windowing = app.service<fggl::display::WindowService>();

	// make a window for our application
	auto* window = windowing->create();
	window->setTitle( "fggl-demo" );
	window->setFullscreen( true );
	app.setWindow(window);

	// load a bunch of modules to provide game functionality
	//app.use<fggl::ecs3::ecsTypes>();
	/*app.use<fggl::gfx::SceneUtils>();
	#ifdef FGGL_MODULE_BULLET
		app.use<FGGL_MODULE_BULLET>();
	#endif*/

	// our test states
	setup_menu(app);
    app.addState<GameScene>("game");
	app.addState<demo::RollBall>("rollball");
	app.addState<demo::TopDown>("topdown");

	return app.run(argc, argv);
}
