/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#include "fggl/gui/fonts.hpp"

namespace fggl::gui {

	FontFace::FontFace(FT_Face face) : m_face(face) {
	}

	FontFace::~FontFace() {
		FT_Done_Face(m_face);
	}

	FontLibrary::FontLibrary(data::Storage* storage) : m_context(nullptr), m_storage(storage) {
		FT_Init_FreeType(&m_context);
	}

	FontLibrary::~FontLibrary() {
		// free all fonts
		for (auto& face : m_cache ){
			face.second = nullptr;
		}

		// shut the library down
		FT_Done_FreeType(m_context);
	}

	GlyphMetrics &FontFace::populateMetrics(char letter) {
		if (FT_Load_Char(m_face, letter, FT_LOAD_RENDER) ) {
			// something bad happened
			return m_metrics['?'];
		}

		GlyphMetrics metrics {
			{ m_face->glyph->bitmap.width, m_face->glyph->bitmap.rows},
			{ m_face->glyph->bitmap_left, m_face->glyph->bitmap_top },
			m_face->glyph->advance.x
		};

		// return the new metrics data
		auto it = m_metrics.emplace(letter, metrics);
		return it.first->second;
	}

	void FontFace::texture(char letter, int &width, int &height, void **buff) {
		if (FT_Load_Char(m_face, letter, FT_LOAD_RENDER) ) {
			// something bad happened
			return;
		}

		width = m_face->glyph->bitmap.width;
		height = m_face->glyph->bitmap.rows;
		*buff = m_face->glyph->bitmap.buffer;
	}
} // namespace fggl::fonts