/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 27/06/22.
//

#ifndef FGGL_AUDIO_NULL_AUDIO_HPP
#define FGGL_AUDIO_NULL_AUDIO_HPP

#include "fggl/audio/audio.hpp"

namespace fggl::audio {

	class NullAudioService : public AudioService {
		public:
			NullAudioService() = default;
			virtual ~NullAudioService() = default;

			void play(const std::string& filename, bool looping = false) override {}
			void play(AudioClip& clip, bool looping = false) override {}
	};

	struct NullAudio {
		constexpr static const char* name = "fggl::audio::NULL";
		constexpr static const std::array<modules::ModuleService, 1> provides = {
			SERVICE_AUDIO_PLAYBACK
		};
		constexpr static const std::array<modules::ModuleService, 0> depends = {};
		static const modules::ServiceFactory factory;
	};

	bool null_factory(modules::ModuleService service, modules::Services& services) {
		if (service == SERVICE_AUDIO_PLAYBACK) {
			services.bind<audio::AudioService, audio::NullAudioService>();
			return true;
		}
		return false;
	}
	const modules::ServiceFactory NullAudio::factory = null_factory;

} // namespace fggl::audio

#endif //FGGL_AUDIO_NULL_AUDIO_HPP
