/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FGGL_GFX_WINDOW_INPUT_HPP
#define FGGL_GFX_WINDOW_INPUT_HPP

#include <memory>
#include <string>

#include "fggl/display/glfw/window.hpp"
#include "fggl/input/input.hpp"

namespace fggl::display::glfw {

	class GlfwInputManager {
		public:
			static GlfwInputManager &instance() {
				static GlfwInputManager *instance = new GlfwInputManager();
				return *instance;
			}

			inline void setup(input::Input* input) {
				m_inputs = input;
			}

			inline void frame() {
				m_inputs->frame(0.0f);
			}

			inline bool alive() {
				return m_inputs != nullptr;
			}

			inline input::MouseInput &mouse() {
				return m_inputs->mouse;
			}

			inline input::KeyboardInput &keyboard() {
				return m_inputs->keyboard;
			}

			inline input::GamepadInput &gamepads() {
				return m_inputs->gamepads;
			}

			inline void onMouseMove(float x, float y) {
				if (m_inputs != nullptr) {
					m_inputs->mouse.axis(input::MouseAxis::X, x);
					m_inputs->mouse.axis(input::MouseAxis::Y, y);
				}
			}

			inline void onMouseScroll(float x, float y) {
				if (m_inputs != nullptr) {
					m_inputs->mouse.axis(input::MouseAxis::SCROLL_X, x);
					m_inputs->mouse.axis(input::MouseAxis::SCROLL_Y, y);
				}
			}

			inline void onMouseButton(int btn, bool state) {
				if (m_inputs != nullptr) {
					m_inputs->mouse.button((fggl::input::MouseButton) btn, state);
				}
			}

			inline void onKeyEvent(int scancode, bool state) {
				if (m_inputs != nullptr) {
					m_inputs->keyboard.set(scancode, state);
				}
			}

		private:
			input::Input* m_inputs;

	};

};

#endif
