/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FGGL_GFX_ATLAS_HPP
#define FGGL_GFX_ATLAS_HPP

#include <fggl/math/types.hpp>
#include <vector>

namespace fggl::gfx {

	struct Bounds2D {
		glm::ivec2 pos;
		glm::ivec2 size;
	};
	bool pack(std::vector<Bounds2D> &bounds);

	template<typename T>
	class ImageAtlas {
		public:
			struct SubImage {
				const T name;
				Bounds2D bounds;
			};

			SubImage &locate(int id) const;
			SubImage &locate(const T &name) const;

			static ImageAtlas<T> *pack(std::vector<SubImage> &images) {
				// extract bounds
				std::vector<Bounds2D> bounds;
				for (auto &image : images) {
					bounds.push_back(image.bounds);
				}

				// allocate the bounds back
				auto result = gfx::pack(bounds);
				for (std::size_t i = 0; i < images.size(); i++) {
					images[i].bounds = bounds[i];
				}

				return new ImageAtlas<T>(images);
			}

		private:
			ImageAtlas(const std::vector<SubImage> &images) : m_images(images) {
			}

			std::vector<SubImage> m_images;
	};

};

#endif
