/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Low-Level Linux-specific path management
// Created by webpigeon on 23/06/22.
// see https://specifications.freedesktop.org/basedir-spec/basedir-spec-latest.html
//

#ifndef FGGL_PLATFORM_LINUX_PATHS_HPP
#define FGGL_PLATFORM_LINUX_PATHS_HPP

#include "fggl/platform/paths.hpp"

#include <filesystem>
#include <array>

#include <vector>

namespace fggl::platform {

	constexpr const char* ENV_USER_CONFIG = "FGGL_CONFIG_HOME";
	constexpr const char* ENV_USER_DATA = "FGGL_DATA_HOME";
	constexpr const char* ENV_USER_CACHE = "FGGL_CACHE_HOME";

	// fallback user paths defined in the XDG spec
	constexpr const char* DEFAULT_USER_CONFIG = "user_config";
	constexpr const char* DEFAULT_USER_DATA = "user_data";

	struct EnginePaths {
		std::filesystem::path userConfig;
		std::filesystem::path userData;
		std::filesystem::path userCache;
	};

	EnginePaths calc_engine_paths(const char* base);

	// search routines for finding data and configuration files
	std::filesystem::path locate_data(const EnginePaths& paths, const std::filesystem::path& relPath);
	std::filesystem::path locate_config(const EnginePaths& paths, const std::filesystem::path& relPath);
	std::filesystem::path locate_cache(const EnginePaths& paths, const std::filesystem::path& relPath);

}

#endif //FGGL_PLATFORM_LINUX_PATHS_HPP
