/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 24/04/22.
//

#ifndef FGGL_PHYS_BULLET_TYPES_HPP
#define FGGL_PHYS_BULLET_TYPES_HPP

#include "fggl/phys/types.hpp"
#include "phys_draw.hpp"

#include <bullet/btBulletDynamicsCommon.h>
#include <bullet/btBulletCollisionCommon.h>

namespace fggl::phys::bullet {

	struct BulletConfiguration {
		btBroadphaseInterface* broadphase;
		btCollisionConfiguration* collisionConfiguration;
		btDispatcher* dispatcher;
		btConstraintSolver* solver;
	};

	/**
	 * Bullet component.
	 */
	struct BulletBody {
		constexpr static const char* name = "phys::bullet::body";
		btMotionState* motion;
		btRigidBody* body;

		inline void release() {
			delete motion;
			delete body;
			motion = nullptr;
			body = nullptr;
		}
	};

	/**
	 * Bullet physics engine implementation.
	 *
	 * This wraps the bullet physics world and provides the systems needed for integrating it into our ECS. The state
	 * is responsible for creating and manging this object.
	 */
	class BulletPhysicsEngine : public PhysicsEngine {
		public:
			explicit BulletPhysicsEngine(entity::EntityManager* world);
			~BulletPhysicsEngine() override;

			void step() override;
			void onEntityDeath(entity::EntityID entity);

			std::vector<ContactPoint> scanCollisions(entity::EntityID entity) override;
			entity::EntityID raycast(math::vec3 from, math::vec3 to) override;
			std::vector<entity::EntityID> raycastAll(math::vec3 from, math::vec3 to) override;
			std::vector<entity::EntityID> sweep(PhyShape& shape, math::Transform& from, math::Transform& to) override;

			bool debugDraw = false;
		private:
			entity::EntityManager* m_ecs;
			BulletConfiguration m_config;
			btDiscreteDynamicsWorld* m_world;
			std::unique_ptr<debug::BulletDebugDrawList> m_debug;
			ContactCache m_contactCache;

			/**
			 * Check for ECS components which aren't in the physics world and add them.
			 */
			void checkForPhys();

			/**
			 * Sync the bullet world state back to the ECS.
			 */
			void forceSyncToECS();

			/**
			 * Deal with physics collisions
			 */
			void pollCollisions();

			// bullet callback functions
			void onCollisionCreate( ContactPoint point );
			void onCollisionProcess( ContactPoint point );
			void onCollisionDestroyed( ContactPoint point );
	};

} // namespace fggl::phys::bullet

#endif //FGGL_PHYS_BULLET_TYPES_HPP
